// Define motor control pins (PWM-capable)
const int motor1Pin = 3; // Ensure these are PWM-capable pins
const int motor2Pin = 5;
const int motor3Pin = 6;
const int motor4Pin = 9;

// Define Hall effect sensor pins
const int hallSensor1Pin = 6;
const int hallSensor2Pin = 7;
const int hallSensor3Pin = 8;
const int hallSensor4Pin = 9;

// Define switch pin
const int switchPin = 10;

// Define magnet counts for each motor
const unsigned long motor1Count = 15;
const unsigned long motor2Count = 20;
const unsigned long motor3Count = 10;
const unsigned long motor4Count = 30;

// Variables to store counts
unsigned long count1 = 0;
unsigned long count2 = 0;
unsigned long count3 = 0;
unsigned long count4 = 0;

// Variable to track previous sensor states
int prevSensor1State = LOW;
int prevSensor2State = LOW;
int prevSensor3State = LOW;
int prevSensor4State = LOW;

// Array to store PWM values for motors
int mot_pwm[] = {55, 255, 255, 255}; // Initial PWM values

// Function to rotate motor until the specified magnet count is reached
void rotateMotor(int motorPin, int sensorPin, unsigned long targetCount, unsigned long &count, int &prevSensorState, int pwmValue) {
  analogWrite(motorPin, pwmValue); // Set motor speed to specified duty cycle
  while (count < targetCount) {
    int sensorState = digitalRead(sensorPin);
    if (sensorState == HIGH && prevSensorState == LOW) {
      count++;
    }
    prevSensorState = sensorState;
  }
  analogWrite(motorPin, 0); // Stop the motor
}

void setup() {
  // Initialize motor pins as outputs
  pinMode(motor1Pin, OUTPUT);
  pinMode(motor2Pin, OUTPUT);
  pinMode(motor3Pin, OUTPUT);
  pinMode(motor4Pin, OUTPUT);

  // Initialize Hall effect sensor pins as inputs
  pinMode(hallSensor1Pin, INPUT);
  pinMode(hallSensor2Pin, INPUT);
  pinMode(hallSensor3Pin, INPUT);
  pinMode(hallSensor4Pin, INPUT);

  // Initialize switch pin as input with pull-up resistor
  pinMode(switchPin, INPUT_PULLUP);

  // Seed the random number generator
  randomSeed(analogRead(0));
}

void loop() {
  // Check if the switch is pressed
  if (digitalRead(switchPin) == LOW) {
    // Reset counts
    count1 = 0;
    count2 = 0;
    count3 = 0;
    count4 = 0;

    // Array of motor and sensor pins
    int motorPins[] = {motor1Pin, motor2Pin, motor3Pin, motor4Pin};
    int sensorPins[] = {hallSensor1Pin, hallSensor2Pin, hallSensor3Pin, hallSensor4Pin};
    unsigned long targetCounts[] = {motor1Count, motor2Count, motor3Count, motor4Count};
    unsigned long *counts[] = {&count1, &count2, &count3, &count4};
    int *prevStates[] = {&prevSensor1State, &prevSensor2State, &prevSensor3State, &prevSensor4State};

    // Update PWM values as needed
    mot_pwm[0] = 55;
    mot_pwm[1] = 255;
    mot_pwm[2] = 255;
    mot_pwm[3] = 255;

    // Rotate each motor with the specified PWM value
    for (int i = 0; i < 4; i++) {
      rotateMotor(motorPins[i], sensorPins[i], targetCounts[i], *counts[i], *prevStates[i], mot_pwm[i]);
    }

    // Wait until the switch is released
    while (digitalRead(switchPin) == LOW);
  }
}

// Function to swap two elements
template <typename T>
void swap(T &a, T &b) {
  T temp = a;
  a = b;
  b = temp;
}
